<?php

namespace Drupal\language_roles\Cache\Context;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\Context\CalculatedCacheContextInterface;
use Drupal\Core\Cache\Context\UserCacheContextBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\language_roles\LanguageRolesManager;

/**
 * Defines the LanguageRolesCacheContext service, for per language role caching.
 */
class LanguageRolesCacheContext extends UserCacheContextBase implements CalculatedCacheContextInterface {

  /**
   * The language roles manager.
   *
   * @var \Drupal\language_roles\LanguageRolesManager
   */
  protected $languageRolesManager;

  /**
   * Constructs a new LanguageRolesCacheContext class.
   *
   * @param \Drupal\Core\Session\AccountInterface $user
   *   The current user.
   * @param \Drupal\language_roles\LanguageRolesManager $language_roles_manager
   *   The language roles manager.
   */
  public function __construct(AccountInterface $user, LanguageRolesManager $language_roles_manager) {
    parent::__construct($user);
    $this->languageRolesManager = $language_roles_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getLabel() {
    return t("User's language roles");
  }

  /**
   * {@inheritdoc}
   */
  public function getContext($role = NULL) {
    $language_roles = $this->languageRolesManager->getUserLanguageRoles($this->user);
    return implode(',', $language_roles);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheableMetadata($role = NULL) {
    return (new CacheableMetadata())->setCacheTags(['user:' . $this->user->id()]);
  }

}
