<?php

declare(strict_types=1);

namespace Drupal\language_roles;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccessPolicyBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\CalculatedPermissionsItem;
use Drupal\Core\Session\RefinableCalculatedPermissionsInterface;

/**
 * Grants permissions based on language roles.
 */
final class LanguageRolesAccessPolicy extends AccessPolicyBase {

  public function __construct(
    protected LanguageRolesManager $languageRolesManager,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function calculatePermissions(AccountInterface $account, string $scope): RefinableCalculatedPermissionsInterface {
    $calculated_permissions = parent::calculatePermissions($account, $scope);

    $language_roles = $this->languageRolesManager->getUserLanguageRoles($account);

    if (!empty($language_roles)) {
      foreach ($this->entityTypeManager->getStorage('user_role')->loadMultiple($language_roles) as $user_role) {
        /** @var \Drupal\user\Entity\Role $user_role */
        $calculated_permissions->addItem(new CalculatedPermissionsItem($user_role->getPermissions(), $user_role->isAdmin()))
          ->addCacheableDependency($user_role);
      }
    }

    return $calculated_permissions;
  }

  /**
   * {@inheritdoc}
   */
  public function getPersistentCacheContexts(): array {
    return ['user.language_roles'];
  }

}
