<?php

declare(strict_types=1);

namespace Drupal\language_roles;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\language\Entity\ConfigurableLanguage;

/**
 * Manager for Language Roles.
 */
final class LanguageRolesManager {

  /**
   * Constructs a LanguageRolesManager object.
   */
  public function __construct(
    private readonly LanguageManagerInterface $languageManager,
    private readonly AccountProxyInterface $currentUser,
    private readonly EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * Get language roles for a user in a language.
   */
  public function getUserLanguageRoles(?AccountInterface $account = NULL, ?ConfigurableLanguage $language = NULL): array {

    $language_roles = [];

    // Use current user if none provided.
    if (!$account) {
      $account = $this->currentUser;
    }

    if (!$account->id()) {
      return $language_roles;
    }

    // Use current language if none provided.
    if (!$language) {
      $language = $this->languageManager->getCurrentLanguage();
    }

    if ($user = $this->entityTypeManager->getStorage('user')->load($account->id())) {
      /** @var \Drupal\user\Entity\User $user */
      if ($value = $user->get('language_roles')->getValue()) {
        foreach ($value as $language_role) {
          if (!empty($language_role)) {
            if ($language_role['language'] == $language->getId() && !in_array($language_role['role'], $language_roles)) {
              $language_roles[] = $language_role['role'];
            }
          }
        }
      }
    }

    sort($language_roles);
    return $language_roles;
  }

}
